@@ -1,60 +1,34 @@
 from OgreInterface.score_function.overlap import SphereOverlap
+from OgreInterface.surface_match.base_surface_matcher import BaseSurfaceMatcher
 from OgreInterface.score_function.generate_inputs import generate_dict_torch
 from OgreInterface.surfaces import Interface
-from pymatgen.io.ase import AseAtomsAdaptor
-from pymatgen.core.periodic_table import Element
-from pymatgen.analysis.local_env import CrystalNN
-from ase.data import atomic_numbers, chemical_symbols
+from ase.data import atomic_numbers
 from typing import Dict, Optional, List
 import numpy as np
-from matplotlib.colors import Normalize
 import matplotlib.pyplot as plt
-from mpl_toolkits.axes_grid1 import make_axes_locatable
-from scipy.interpolate import RectBivariateSpline, CubicSpline
-from copy import deepcopy
-from itertools import groupby, combinations_with_replacement, product
+from scipy.interpolate import RectBivariateSpline
 from ase import Atoms
 
 
-class SphereSurfaceMatcher:
+class SphereSurfaceMatcher(BaseSurfaceMatcher):
     def __init__(
         self,
         interface: Interface,
         radius_dict: Optional[Dict[str, float]] = None,
-        grid_density_x: int = 15,
-        grid_density_y: int = 15,
-        xlim: List[float] = [0.0, 1.0],
-        ylim: List[float] = [0.0, 1.0],
+        grid_density: float = 0.4,
     ):
-        self.xlim = xlim
-        self.ylim = ylim
-        self.interface = interface
-        self.matrix = deepcopy(interface._orthogonal_structure.lattice.matrix)
-        self._vol = np.linalg.det(self.matrix)
-
-        if self._vol < 0:
-            self.matrix *= -1
-            self._vol *= -1
+        super().__init__(
+            interface=interface,
+            grid_density=grid_density,
+        )
 
         self.radius_dict = self._get_radii(radius_dict)
         self.cutoff = self._get_cutoff()
         self.d_interface = self.interface.interfacial_distance
         self.film_part = self.interface._orthogonal_film_structure
         self.sub_part = self.interface._orthogonal_substrate_structure
-        self.grid_density_x = grid_density_x
-        self.grid_density_y = grid_density_y
         self.opt_xy_shift = np.zeros(2)
 
-        self.shifts, self.X, self.Y = self._generate_shifts()
-        self.z_PES_data = None
-
-    def get_optmized_structure(self):
-        opt_shift = self.opt_xy_shift
-
-        self.interface.shift_film_inplane(
-            x_shift=opt_shift[0], y_shift=opt_shift[1], fractional=True
-        )
-
     def _get_radii(self, radius_dict):
         sub_radii = radius_dict["sub"]
         film_radii = radius_dict["film"]
@@ -71,15 +45,6 @@ class SphereSurfaceMatcher:
 
         return cutoff_val
 
-    def _generate_shifts(self):
-        grid_x = np.linspace(self.xlim[0], self.xlim[1], self.grid_density_x)
-        grid_y = np.linspace(self.ylim[0], self.ylim[1], self.grid_density_y)
-        X, Y = np.meshgrid(grid_x, grid_y)
-
-        shifts = np.c_[X.ravel(), Y.ravel()]
-
-        return shifts, X, Y
-
     def _get_shifted_atoms(self, shifts: np.ndarray) -> List[Atoms]:
         sub_atoms = self.interface.get_substrate_supercell(return_atoms=True)
         sub_atoms.set_array("is_film", np.zeros(len(sub_atoms)).astype(bool))
@@ -131,76 +96,32 @@ class SphereSurfaceMatcher:
 
         return overlap
 
-    def _get_interpolated_data(self, X, Y, Z):
-        x_grid = np.linspace(self.xlim[0], self.xlim[1], self.grid_density_x)
-        y_grid = np.linspace(self.ylim[0], self.ylim[1], self.grid_density_y)
+    def _get_interpolated_data(self, Z, image):
+        x_grid = np.linspace(0, 1, self.grid_density_x)
+        y_grid = np.linspace(0, 1, self.grid_density_y)
         spline = RectBivariateSpline(y_grid, x_grid, Z)
 
-        x_grid_interp = np.linspace(self.xlim[0], self.xlim[1], 401)
-        y_grid_interp = np.linspace(self.ylim[0], self.ylim[1], 401)
+        x_grid_interp = np.linspace(0, 1, 101)
+        y_grid_interp = np.linspace(0, 1, 101)
 
         X_interp, Y_interp = np.meshgrid(x_grid_interp, y_grid_interp)
         Z_interp = spline.ev(xi=Y_interp, yi=X_interp)
-        frac_shifts = np.c_[
-            X_interp.ravel(),
-            Y_interp.ravel(),
-            np.zeros(X_interp.shape).ravel(),
-        ]
+        frac_shifts = (
+            np.c_[
+                X_interp.ravel(),
+                Y_interp.ravel(),
+                np.zeros(X_interp.shape).ravel(),
+            ]
+            + image
+        )
 
-        cart_shifts = frac_shifts.dot(self.matrix)
+        cart_shifts = frac_shifts.dot(self.shift_matrix)
 
         X_cart = cart_shifts[:, 0].reshape(X_interp.shape)
         Y_cart = cart_shifts[:, 1].reshape(Y_interp.shape)
 
         return X_cart, Y_cart, Z_interp
 
-    def _plot_heatmap(
-        self, fig, ax, X, Y, Z, borders, cmap, fontsize, show_max
-    ):
-        ax.set_xlabel(r"Shift in $x$ ($\AA$)", fontsize=fontsize)
-        ax.set_ylabel(r"Shift in $y$ ($\AA$)", fontsize=fontsize)
-
-        im = ax.contourf(
-            X,
-            Y,
-            Z,
-            cmap=cmap,
-            levels=200,
-            norm=Normalize(vmin=np.nanmin(Z), vmax=np.nanmax(Z)),
-        )
-
-        ax.plot(
-            borders[:, 0],
-            borders[:, 1],
-            color="black",
-            linewidth=2,
-        )
-
-        divider = make_axes_locatable(ax)
-        cax = divider.append_axes("top", size="5%", pad=0.1)
-        cbar = fig.colorbar(im, cax=cax, orientation="horizontal")
-        cbar.ax.tick_params(labelsize=fontsize)
-        cbar.ax.locator_params(nbins=3)
-
-        if show_max:
-            E_max = np.max(Z)
-            label = (
-                "$E_{adh}$ (eV/$\\AA^{2}$) : "
-                + "$E_{max}$ = "
-                + f"{E_max:.4f}"
-            )
-            cbar.set_label(label, fontsize=fontsize)
-        else:
-            label = "$E_{adh}$ (eV/$\\AA^{2}$)"
-            cbar.set_label(label, fontsize=fontsize)
-
-        cax.xaxis.set_ticks_position("top")
-        cax.xaxis.set_label_position("top")
-        ax.tick_params(labelsize=fontsize)
-        ax.set_xlim(borders[:, 0].min(), borders[:, 0].max())
-        ax.set_ylim(borders[:, 1].min(), borders[:, 1].max())
-        ax.set_aspect("equal")
-
     def run_surface_matching(
         self,
         cmap: str = "jet",
@@ -220,29 +141,24 @@ class SphereSurfaceMatcher:
         sub_overlap = overlap[0]
         film_overlap = overlap[1]
 
-        interface_overlap = overlap[2:].reshape(self.X.shape)
+        x_grid = np.linspace(0, 1, self.grid_density_x)
+        y_grid = np.linspace(0, 1, self.grid_density_y)
+        X, Y = np.meshgrid(x_grid, y_grid)
+
+        interface_overlap = overlap[2:].reshape(X.shape)
 
         Z = (
             sub_overlap + film_overlap - interface_overlap
         ) / self.interface.area
 
-        X_plot, Y_plot, Z_overlap = self._get_interpolated_data(
-            self.X, self.Y, Z
-        )
-
         a = self.matrix[0, :2]
         b = self.matrix[1, :2]
-        borders = np.vstack(
-            [
-                self.xlim[0] * a + self.ylim[0] * b,
-                self.xlim[1] * a + self.ylim[0] * b,
-                self.xlim[1] * a + self.ylim[1] * b,
-                self.xlim[0] * a + self.ylim[1] * b,
-                self.xlim[0] * a + self.ylim[0] * b,
-            ]
-        )
+
+        borders = np.vstack([np.zeros(2), a, a + b, b, np.zeros(2)])
+
         x_size = borders[:, 0].max() - borders[:, 0].min()
         y_size = borders[:, 1].max() - borders[:, 1].min()
+
         ratio = y_size / x_size
 
         if ratio < 1:
@@ -256,36 +172,23 @@ class SphereSurfaceMatcher:
             figsize=(figx, figy),
             dpi=dpi,
         )
-        self._plot_heatmap(
+
+        max_Z = self._plot_surface_matching(
             fig=fig,
             ax=ax,
-            X=X_plot,
-            Y=Y_plot,
-            Z=Z_overlap,
-            borders=borders,
+            X=X,
+            Y=Y,
+            Z=Z,
+            dpi=dpi,
             cmap=cmap,
             fontsize=fontsize,
             show_max=show_max,
+            shift=True,
         )
 
-        frac_shifts = np.c_[
-            X_plot.ravel(), Y_plot.ravel(), np.zeros(Y_plot.shape).ravel()
-        ].dot(np.linalg.inv(self.matrix))
-        opt_shift = frac_shifts[np.argmax(Z_overlap.ravel())]
-        max_Z = np.max(Z_overlap)
-        plot_shift = opt_shift.dot(self.matrix)
-
-        ax.scatter(
-            [plot_shift[0]],
-            [plot_shift[1]],
-            fc="white",
-            ec="black",
-            marker="X",
-            s=100,
-            zorder=10,
-        )
-
-        self.opt_xy_shift = opt_shift[:2]
+        ax.set_xlim(borders[:, 0].min(), borders[:, 0].max())
+        ax.set_ylim(borders[:, 1].min(), borders[:, 1].max())
+        ax.set_aspect("equal")
 
         fig.tight_layout()
         fig.savefig(output, bbox_inches="tight")
